<?php
class MailSettingException extends Exception 
{
	public function __construct()
	{
		parent::__construct('Mail Setting Error!');
	}
}

class MailSendingException extends Exception 
{
	public function __construct()
	{
		parent::__construct('Mail Sending Error!');
	}
}

class MRUMailer
{
	private $userName;
	private $passwd;
	private $port;
	private $serverAddr;
	private $isSSL;
	//private $isTSL;
	private $needAuth = 1;
	private $swift = null;
	
	const HelloMsg = 'Hello';
	const MailAutoMsg = 'This mail is automatically sent out by Marvell RAID Utility.Please don\'t reply to this mail.';
	const MailHeadMsg = 'The following event(s) have ocurred on the machine';
	
	public function getUserName()
	{
		return $this->userName;
	}
	public function setUserName($userName)
	{
		$this->userName = $userName;
	}
	
	public function getPasswd()
	{
		return $this->passwd;
	}
	public function setPasswd($passwd)
	{
		$this->passwd = $passwd;
	}
	
	public function getPort()
	{
		return $this->port;
	}
	public function setPort($port)
	{
		$this->port = $port;
	}
	
	public function getServerAddr()
	{
		return $this->serverAddr;
	}
	public function setServerAddr($serverAddr)
	{
		$this->serverAddr = $serverAddr;
	}
	
	public function isSSL()
	{
		return $this->isSSL;
	}
	public function setSSL($isSSL)
	{
		$this->isSSL = $isSSL;
	}
	
	public function needAuth()
	{
		$this->needAuth = 1;
	}
	
	public function notNeedAuth()
	{
		$this->needAuth = 0;
	}
	
	//get smtp connection
	private function getSMTPConn()
	{
		$smtp = null;
		
		if ($this->isSSL)
		{
			//use ssl
			$smtp = new Swift_Connection_SMTP($this->serverAddr,$this->port,
				Swift_Connection_SMTP::ENC_TLS);
		}
		else if ( ! $this->port ) {
			//use default port no : 25
			$smtp = new Swift_Connection_SMTP($this->serverAddr);
		}
		else 
		{
			$smtp = new Swift_Connection_SMTP($this->serverAddr,$this->port);
		}
		
		//if ($this->needAuth) {
		if ($this->userName) {
			//if user name is set,then need authentication
			$smtp->setUsername($this->userName);
			$smtp->setPassword($this->passwd);
		}
		//}
		
		$smtp->setTimeout(30);
		
		return $smtp;
	}
	
	//
	public function initMailer()
	{
		try {
			$smtp = $this->getSMTPConn();
			$this->swift = new Swift($smtp);
			//$this->swift->log->enable();
		}
		catch (Exception $e)
		{
			throw new MailSettingException();
		}
	}
	
	//send basic mail
	public function send($to,$subject,$content,$type='text/plain')
	{
		try {
			$message = new Swift_Message($subject,$content,$type);
			$this->swift->send($message,$to,$this->userName);
		}
		catch (Exception $e)
		{
			throw new MailSendingException();
		}
	}
	
	public function sendTestMail()
	{
		try {
			$message = new Swift_Message(gettext('Email Setting Test'));
			$txtMessagePart = new Swift_Message_Part(gettext(self::MailAutoMsg),'text/plain');
			$htmlMessagePart = new Swift_Message_Part(gettext(self::MailAutoMsg),'text/html');
			$message->attach($txtMessagePart);
			$message->attach($htmlMessagePart);
			$recipients = new Swift_RecipientList();
			if ($this->userName) {
				$recipients->addTo($this->userName,$this->userName);
				$this->swift->send($message,$recipients,$this->userName);
			}
			/*
			else 
			{
				$recipients->addTo('noreply@mru.com',"MRU");
				$this->swift->send($message,$recipients,'noreply@mru.com');
			}
			*/
			
		}
		catch (Exception $e)
		{
			throw new MailSendingException();
		}
	}
	
	//send event notify mail(text/html)
	//$contacts = array("mailaddress@mail.com"=>userName,.......);
	public function sendEventNotify($contacts,$eventsRelated)
	{
		if ($this->swift != null && $contacts && $eventsRelated) {
			//if swift are successfully initialized, both contacts and events are not empty
			//then send mails
			$recipients = new Swift_RecipientList();
			/*
			if (!$this->userName) {
				//if no user name,default user name
				$recipients->addTo("noreply@mru.com","MRU");
			}
			else
			{
				$recipients->addTo($this->userName,$this->userName);
			}
			*/
			foreach ($contacts as $mailAddr=>$userName)
			{
				$recipients->addTo($mailAddr,$userName);
			}
			
			$message = $this->formMultipartContent(gettext("MRU User"),$eventsRelated);
			try {
				//$this->swift->send($message,$recipients,"noreply@mru.com");
				
				if (!$this->userName) {
					//if no user name,default user name
					$this->swift->send($message,$recipients,"noreply@mru.com");
				}
				else
				{
					$this->swift->send($message,$recipients,$this->userName);
				}
				
			}
			catch (Exception $e)
			{
				//echo $e->getMessage() .'<br/>';
				//echo '<br/>'  . $this->swift->log->dump() . '<br/>';
				throw new MailSendingException();
			}
		}
	}
	
	private function formMultipartContent($name,$eventsRelated)
	{
		$message = new Swift_Message(gettext('Marvell RAID Utility Event Notification'));
		
		$os_string = getenv('OS');
		$newLine = "\n";
		if (stripos($os_string,'win') !== false) 
		{
			$newLine = "\r\n";
		}
		
		$htmlNewLine = '<br/>';
		
		$txtContent = '';
		$htmlContent = '';
		
		/*
		$htmlContent .= '<html>';
		$htmlContent .= '<head>';
		$htmlContent .= '<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=us-ascii" />';
		$htmlContent .= '<style></style>';
		$htmlContent .= '</head>';
		$htmlContent .= '<body>';
		$htmlContent .= '</body>';
		$htmlContent .= '</html>';
		*/
		
		$htmlContent .= gettext(self::HelloMsg) .', ' . $name .'.' . $htmlNewLine;
		$htmlContent .= gettext(self::MailAutoMsg) . $htmlNewLine;
		$htmlContent .= gettext(self::MailHeadMsg).'('
			. getenv('computername') .').' . $htmlNewLine;
		
		$txtContent = gettext('Hello') .', ' . $name .'.' . $newLine;
		$txtContent .= gettext(self::MailAutoMsg) . $newLine;
		$txtContent .= gettext(self::MailHeadMsg).'('
			. getenv('computername') .').' . $newLine;
		
		foreach ($eventsRelated as $event)
		{
				$htmlContent .= $event->Date . " : [" 
					. $event->SeverityTitle . "] " . $event->Class . " : " . $event->Log . $htmlNewLine;
				$txtContent .=  $event->Date . " : [" 
					. $event->SeverityTitle . "] " . $event->Class . " : " . $event->Log . $newLine;
		}
		
		$txtMessagePart = new Swift_Message_Part($txtContent,'text/plain');
		//don't use the encoding here. for some mail server don't support UTF-8 what we are using.
		$htmlMessagePart = new Swift_Message_Part($htmlContent,'text/html');
		
		$message->attach($txtMessagePart);
		$message->attach($htmlMessagePart);
		
		return $message;
	}
}
?>